/*****************************************************
Chip type           : ATmega8
*****************************************************/

#include <mega8.h>
#include <delay.h>
#include <math.h>
// Alphanumeric LCD Module functions
#asm
   .equ __lcd_port=0x12 ;PORTD
#endasm
#include <lcd.h>

// Declare your global variables here  
#define light_on PORTB.0=1;
#define light_off PORTB.0=0;

unsigned char Ok=27, Down=29, Up=30, key=31;

const float P2=6.283185308;
const float DR=0.017453293;
const float K1=0.262516090;

const unsigned char days_month[12]={31,28,31,30,31,30,31,31,30,31,30,31};
 
unsigned char  hr, mins, sec, msec, cc = 0,  day=8, B5=69, C0, H=3, lat=33, cor=10; // (kor=10  )

long int J3, Y=2009, M=3;

long int MM[2], HH[2];

float aA[2], aD[2];

float D, T, T0, TT, A, B, A0, D0, D1, E, F, J, S, L, G, V, U, V0, V1, V2; 

bit M8, W8, st;



char delim=':';

// Declare your global variables here

// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{ 
  cc++;
  if (cc>=1) {msec++; cc=0;};//   !!!
    // Place your code here
}

// Timer 2 overflow interrupt service routine
interrupt [TIM2_OVF] void timer2_ovf_isr(void)
{
  key=PINC & 0b00011111;//  
}

void me_delay(void)
{
  delay_ms(200); 
  key=0;
}

void cld_cls(void)
{
  lcd_gotoxy(0,0);
  lcd_putsf("                ");//16 
  HH[0]=0; MM[0]= 0; //
  HH[1]=0; MM[1]= 0; //
}

// Declare your global variables here

float Abs(float x)
{
  return( x<0 ?-x:x);
}


int Sgn(float x)
{  
  return( x>=0 ?1:-1);
}

long int Trunc (float x)
{
  if (x>=0) {x=floor(x);}
  else {x=ceil(x);}
  return (long int)x;
}

void sun_info()          
{ int i;
  for (i=0; i<2; i++)
  {
    lcd_gotoxy(6*i,0);
    lcd_putchar (HH[i]/10%10+0x30);
    lcd_putchar (HH[i]%10+0x30);
    lcd_putchar (':');
    lcd_putchar (MM[i]/10%10+0x30);  //+kor
    lcd_putchar (MM[i]%10+0x30);     //+kor
  }
}

void Calendar(void)        //G,Y,D,D1,J,M,F,S,J3
{
//  G = 1;
//  if (Y < 1583) {G = 0;};//2009
  D1 = Trunc(D);
  F = D - D1 - 0.5; 
  J = -Trunc(7 * (Trunc((M + 9) / 12) + Y) / 4);                                      
//  if (G == 1)
//  {
    S = Sgn(M - 9);
    //A = Abs(M - 9);
    J3 = Trunc(Y + S * Trunc(Abs(M - 9)/7)); //A / 7));
    J3 =( -(((J3 / 100) + 1) * 3 / 4));
//  }
  J = J + Trunc(275 * M / 9) + D1 + J3;
  J = J + 1721029 + 367 * Y;
  if (F >= 0) {goto end1;};
  F = F + 1;
  J = J - 1;
end1:
}

void GetTimeZone(void)       //S,T0,T,lat,H,DR
{
  //T0 = T / 36525;
  S = 24110.5 + 8640184.813 * T / 36525;//T0;
  
  S = S -(86636.6 * (float) H / 24) + 86400 * lat/ 360;//L5;
  S = S / 86400;
  S = S - Trunc(S);
  T0 = S * 360 * DR;
} 

float GetAtan()
{
  return atan(S / sqrt(1 - S * S));
}

void PosOfSun(char ind)        //(L,G,V,U),P2,T,TT,S
{

  //       
  //  (Van Flandern & Pulkkinen, 1979)
  L = 0.779072 + 0.00273790931 * T;
  G = 0.993126 + 0.0027377785 * T;
  L = L - Trunc(L);
  G = G - Trunc(G);
  L = L * P2;
  G = G * P2;
  V = 0.39785 * sin(L);
  V = V - 0.01000 * sin(L - G);
  V = V + 0.00333 * sin(L + G);
  V = V - 0.00021 * TT * sin(L); 
  
  U = 1 - 0.03349 * cos(G);
  U = U - 0.00014 * cos(2 * L);
  U = U + 0.00008 * cos(L);    
  
  S = -0.00010 - 0.04129 * sin(2 * L);
  S = S + 0.03211 * sin(G);
  S = S + 0.00104 * sin(2 * L - G);
  S = S - 0.00035 * sin(2 * L + G);
  S = S - 0.00008 * TT * sin(G);
  S = S / sqrt(U - V * V);
  
  aA[ind] = L + GetAtan();//A5;
  S = V / sqrt(U);
  aD[ind] = GetAtan();
}

float getV(float vd, float vh)
{
  return (S * sin(vd) + cos(B5 * DR)* cos(vd) * cos(vh) - cos(DR * 90.833));   //Z;//C * cos(vd) * cos(vh) - Z;
}

void PossibleEvents(int Hour)       //KL,A0,L,G,Hour,V0,V1,V2,A,B,D,E
{
  //P = ((float)Hour + 1) / 24;
  L = aA[0] + ((float)Hour + 1) / 24 * (aA[1] - aA[0]); // A2 = //P         //DA;
  G = aD[0] + ((float)Hour + 1) / 24 * (aD[1] - aD[0]); // D2 =         //DD;
  st = 0;
                                             //L0 = T0 + Hour * K1;
                                             //L2 = T0 + Hour * K1 + K1;
  A = T0 + Hour * K1 - A0;                   //H0 =
  B = T0 + Hour * K1 + K1 - L;               //H2 = //L2 - A2;
                                             //H1 = (H2 + H0) / 2; //  ,
                                             //D1 = (D2 + D0) / 2; //   
  if (Hour <= 0) {V0 = getV(D0, A);}         //S * sin(D0) + C * cos(D0) * cos(H0) - Z;}
  V2 = getV(G,B);                            //S * sin(D2) + C * cos(D2) * cos(H2) - Z;
  if (Sgn(V0) == Sgn(V2)) {goto end2;}
  V1 = getV((G + D0) / 2,(B + A) / 2);      //S * sin(D1) + C * cos(D1) * cos((H2 + H0) / 2) - Z;//cos(H1) - Z;
  A = 2 * V2 - 4 * V1 + 2 * V0;
  B = 4 * V1 - 3 * V0 - V2;
  D = B * B - 4 * A * V0;
  if (D < 0) {goto end2;};
  D = sqrt(D);
  E = (-B + D) / (2 * A);
  if ((E > 1) || (E < 0)) {E = (-B - D) / (2 * A);}
                                            //T3 = Hour + E + 1 / 120; // 
  //A = Trunc(Hour + E + 1 / 120);
          //H3 =  
  A = (Hour + E + 1 / 120 );//+ (float)kor/60;//+ kor / 60;  ////// 0.2
  B = Trunc((A - Trunc(A)) * 60); //M3
  if ((V0 < 0) && (V2 > 0)) {HH[0]=Trunc(A);  MM[0]=Trunc(B);  M8 = 1; st = 1;};    //?????
  if ((V0 > 0) && (V2 < 0)) {HH[1]=Trunc(A);  MM[1]=Trunc(B);  W8 = 1; st = 1;};    //?????
end2:   
  if (st != 0) {sun_info();}
  A0 = L;
  D0 = G;
  V0 = V2;
}


void OutInform(void)
{
 if (((M8 == 0) && (W8 == 0)) && ((V2 < 0) || (V2 > 0)) || ((M8 == 0) || (W8 == 0))) {st=0; cld_cls();}
}

void show_info(void)             //D,day,T,J,TT,F,H,S,Z
{ 
  D = day;
  //L5=lat;
  //L5 = lat/ 360;
  //Z0 = (float) H / 24;
  Calendar();
  T = (J - 2451545) + F;
  TT = T / 36525 + 1; // TT - ,   1900.0
  GetTimeZone(); //   
  T = T - (float) H / 24;
  PosOfSun(0); //   
  T = T + 1;
  PosOfSun(1); 
  if (aA[1] < aA[0]) {aA[1] = aA[1] + P2;}
  S = sin(B5 * DR);
  M8 = 0;
  W8 = 0;
  A0 = aA[0];
  D0 = aD[0];
  
  for (C0 = 0; C0 < 24; C0++)
  {PossibleEvents(C0);}
   OutInform();
}

void display(void)
{ 
  lcd_gotoxy(13,0); //13
  lcd_putsf("Z");
  lcd_putchar (H/10%10+0x30);
  lcd_putchar (H%10+0x30);
  lcd_gotoxy(0,1);
  lcd_putchar (hr/10%10+0x30);
  lcd_putchar (hr%10+0x30);
  lcd_putchar (delim);
  lcd_putchar (mins/10%10+0x30);
  lcd_putchar (mins%10+0x30);
  lcd_putsf (" ");
  lcd_putchar ((day/10)%10+0x30);
  lcd_putchar (day%10+0x30);
  lcd_putsf ("/");
  lcd_putchar ((M/10)%10+0x30);
  lcd_putchar (M%10+0x30);
  lcd_putsf ("/");
  lcd_putchar ((Y/1000)%10+0x30);
  lcd_putchar ((Y/100)%10+0x30);
  lcd_putchar ((Y/10)%10+0x30);
  lcd_putchar (Y%10+0x30);
}


void clock(void)
{ unsigned char qty_days;
  if (msec>=1) //150
  {
    msec=0; sec++; 
    if (delim==':') {delim=' ';} 
    else {delim=':';};
  }; 
  if (sec==60) {sec=0; mins++;};  //   ,    60
  if (mins==60) {mins=0; hr++;}; //   ,    60
  if (hr==24) {day++; hr=0; show_info();};//   24
  qty_days=days_month[M-1];//   
  if ((M==2) & (Y%4)==(Y/4)) {qty_days=29;}//   
  if (day>qty_days) {M++; day=1;};//   
  if (M>12) {M=1; Y++;};//   12 
  
  display();
 
 }
    
 void on_of_led(void)
  {  
    int HC[2], MC[2];
    long int time_real;
    time_real=(HH[0]*60 + MM[0])+cor;
    HC[0]=Trunc(time_real/60);
    if (HC[0]>24) {HC[0] = HC[0] - 24;};
    MC[0]=(int)(time_real - HC[0]*60);

    time_real=(HH[1]*60 + MM[1]) - cor;
    HC[1]=Trunc(time_real/60);
    if (HC[1]<0) {HC[1]=(int)Abs(HC[1]);};
    MC[1]=(int)(time_real - HC[1]*60);
  
    if ((hr==HC[0]) && (mins==MC[0])) {light_off;}  //   
    if ((hr==HC[1]) && (mins==MC[1])) {light_on;}   //   
  }
 
void keypress(unsigned char ind, long int ddey, int ga, int gb, char sw) 
{
  while (key!=Ok)
  { 
    if (sw==1)
    {
      lcd_gotoxy(6,1);
      lcd_putchar (ddey/10%10+0x30);
      lcd_putchar (ddey%10+0x30);
    }
    else 
    {  
      display();
    } 
    if ((key==Up) & (ddey<ga)) {ddey++; delay_ms(200);}
    if ((key==Down) & (ddey>gb)) {ddey--; delay_ms(200);}
   
    if (ind==1) {B5=ddey;}   
    if (ind==2) {lat=(float)ddey;}
    if (ind==3) {M=ddey;}
    if (ind==4) {day=ddey;}
    if (ind==5) {Y=ddey;}
    if (ind==6) {hr=ddey;}  
    if (ind==7) {mins=ddey;} 
    if (ind==8) {cor=ddey;} 
  }
  me_delay();
  lcd_clear();////////
  lcd_gotoxy(0,0);  
}



void main(void)
{

PORTB=0x00;
DDRB=0x01;

PORTC=0x1F;
DDRC=0x00;

PORTD=0x00;
DDRD=0xF7;

TCCR0=0x02;
TCNT0=0x00;
//OCR0=0x00;///////   

TCCR1A=0x00;
TCCR1B=0x00;//5
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

ASSR=0x00;
TCCR2=0x07;//3
TCNT2=0x00;
OCR2=0x00;

MCUCR=0x00;

TIMSK=0x41;//00

ACSR=0x80;
SFIOR=0x00;

/// LCD module initialization
lcd_init(16);

#asm("sei")


lcd_gotoxy(0,0);
lcd_putsf("AstroTimer v.1.0");

while (key!=Ok){}
me_delay();
lcd_clear(); 

//lcd_gotoxy(6,0);        
lcd_putsf("Latitude"); 
keypress(1, B5, 90, 0, 1); 
lcd_clear();

//lcd_gotoxy(6,0);
lcd_putsf("Longitude"); //Dolg
keypress(2, lat, 180, 0, 1); 
lcd_clear();

//lcd_gotoxy(4,0);
lcd_putsf("Cor +- mins");
keypress(8, cor, 59, 0, 1);  //
lcd_clear();

//lcd_gotoxy(0,0);
lcd_putsf("Month");
keypress(3, M, 12, 1, 0); 
lcd_clear();

lcd_putsf("Day");
keypress(4,day,days_month[M], 1, 0); 

lcd_putsf("Year");
keypress(5, Y, 2100, 2000, 0); 

lcd_putsf("Hour"); 
keypress(6, hr, 24, 1, 0); 

lcd_putsf("Min");
keypress(7,mins, 59, 1, 0);
 

 
lcd_putsf("Zone"); 
while (key!=Ok)
{
  display(); 
  if ((key==Up) & (H<12)) {H++; delay_ms(200);}
  if ((key==Down) & (H>0)) {H--; delay_ms(200);}
  // 0  ()   180   -  
}

lcd_clear();
clock();
show_info();

while (1)
      {
      // Place your code here
          clock(); 
          on_of_led();
      };
}
